#	pyradium - HTML presentation/slide show generator
#	Copyright (C) 2021-2022 Johannes Bauer
#
#	This file is part of pyradium.
#
#	pyradium is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	pyradium is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with pyradium; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import os
import json
from .Exceptions import ConfigurationException

class GlobalConfig():
	def __init__(self, filename):
		self._filename = filename
		try:
			with open(self._filename) as f:
				self._config = json.load(f)
		except FileNotFoundError:
			self._config = { }
		self._plausibilize()

	@classmethod
	def read(cls):
		filename = os.path.expanduser("~/.config/pyradium/configuration.json")
		return cls(filename)

	def _plausibilize(self):
		if self.has("spellcheck", "jar") and self.has("spellcheck", "uri"):
			raise ConfigurationException("%s: spellcheck.jar and spellcheck.uri are mutually exclusive" % (self._filename))

	def has(self, *args):
		value = self._config
		for key in args:
			if key not in value:
				return False
			value = value[key]
		return True

	def get(self, *args):
		value = self._config
		for key in args:
			value = value.get(key)
			if value is None:
				return value
		return value



#	pyradium - HTML presentation/slide show generator
#	Copyright (C) 2015-2022 Johannes Bauer
#
#	This file is part of pyradium.
#
#	pyradium is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	pyradium is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with pyradium; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import sys
import logging
from pyradium.FriendlyArgumentParser import FriendlyArgumentParser

class BaseModifyCommand():
	_SUBCOMMANDS = { }
	_NAME = None
	_DESCRIPTION = None
	_VISIBLE = True

	def __init__(self, args):
		self._args = args

	@classmethod
	def get_supported_cmd_list(cls):
		return cls._SUBCOMMANDS.keys()

	@classmethod
	def get_handler(cls, name):
		return cls._SUBCOMMANDS[name]

	@classmethod
	def register(cls, handler):
		if not handler._VISIBLE:
			return
		name = handler._NAME
		if name is None:
			raise ValueError("Handler name not set in child class.")
		if handler._DESCRIPTION is None:
			raise ValueError("Handler description not set in child class.")

		cls._SUBCOMMANDS[handler._NAME] = handler

	@classmethod
	def _gen_parser(cls, parser):
		raise NotImplementedError("_gen_parser")

	@classmethod
	def parse(cls, arguments):
		parser = FriendlyArgumentParser(description = cls._DESCRIPTION, prog = f"{sys.argv[0]} {cls._NAME}", add_help = False)
		cls._gen_parser(parser)
		parsed_args = parser.parse_args(arguments)
		logging.getLoggerClass().set_logging_by_verbosity(parsed_args.verbose)
		return parsed_args



#	pyradium - HTML presentation/slide show generator
#	Copyright (C) 2015-2022 Johannes Bauer
#
#	This file is part of pyradium.
#
#	pyradium is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	pyradium is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with pyradium; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import os
import re
import xml.dom.minidom
from .Tools import XMLTools
from .Acrofile import Acrofile
from .BaseAction import BaseAction

class ActionAcroScan(BaseAction):
	_WHITESPACE_RE = re.compile(r"(\s+)")

	def _slide_callback(self, node):
		if node.nodeType == node.TEXT_NODE:
			split_text = self._WHITESPACE_RE.split(node.wholeText)
			display_text = node.wholeText.replace("\t", "  ").replace("\n", " ")
			performed_replacement = False

			processed_split_text = [ ]
			for word in split_text:
				if self._acrofile.has(word):
					acro = self._acrofile.get(word)
					print("Found possible acronym on slide %d:" % (self._slideno))
					print("%s = %s" % (word, acro["text"]))
					print("Context: %s" % (display_text))
					yn = input("Replace (y/n)? ")
					print()
					if yn.lower() == "y":
						self._performed_change = True
						performed_replacement = True
						processed_split_text.append([ word ])
					else:
						processed_split_text.append(word)
				else:
					processed_split_text.append(word)

			if performed_replacement:
				replacement_nodes = [ ]
				for element in processed_split_text:
					if isinstance(element, str):
						replacement_nodes.append(node.ownerDocument.createTextNode(element))
					else:
						ac_node = node.ownerDocument.createElement("s:ac")
						ac_node.appendChild(node.ownerDocument.createTextNode(element[0]))
						replacement_nodes.append(ac_node)
				XMLTools.replace_node(node, replacement_nodes)

	def _process_slide(self, slide):
		XMLTools.walk(slide, self._slide_callback, cancel_descent_predicate = lambda node: (node.nodeType == node.ELEMENT_NODE) and (node.tagName in [ "s:ac", "s:tex", "s:code", "s:term", "s:var" ]))

	def run(self):
		self._acrofile = Acrofile.load_from_file(self._args.acrofile)
		dom = xml.dom.minidom.parse(self._args.infile)
		pres = XMLTools.findall(dom, "presentation")[0]
		slides = XMLTools.findall(pres, "slide")
		self._performed_change = False
		for (slideno, slide) in enumerate(slides, 1):
			self._slideno = slideno
			self._process_slide(slide)

		if self._performed_change:
			while True:
				tempfile_name = self._args.infile + "_" + os.urandom(8).hex()
				if not os.path.exists(tempfile_name):
					break
			with open(tempfile_name, "w") as f:
				dom.writexml(f)
			os.rename(tempfile_name, self._args.infile)



#	pyradium - HTML presentation/slide show generator
#	Copyright (C) 2015-2023 Johannes Bauer
#
#	This file is part of pyradium.
#
#	pyradium is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	pyradium is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with pyradium; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import json
import logging
import collections
import enum
import os
import contextlib

_log = logging.getLogger(__spec__.name)

class SpellcheckExceptionType(enum.IntEnum):
	GeneralWord = 0
	RuleWord = 1
	RuleContext = 2

class SpellcheckDictionary():
	def __init__(self, filename, initially_create = True):
		self._filename = filename
		self._modified = False

		if initially_create:
			try:
				with open(self._filename) as f:
					self._entries = json.load(f, object_pairs_hook = collections.OrderedDict)
			except FileNotFoundError:
				self._entries = collections.OrderedDict()
				self.write()
				_log.info("Creating empty dictionary: %s", self._filename)
			except json.decoder.JSONDecodeError as e:
				self._entries = collections.OrderedDict()
				self.write()
				_log.warning("Creating empty dictionary because it was malformed JSON: %s (%s)", self._filename, str(e))
		else:
			self._entries = collections.OrderedDict()

	@classmethod
	def open_global_dict(cls):
		directory = os.path.realpath(os.path.expanduser("~/.config/pyradium"))
		with contextlib.suppress(FileExistsError):
			os.makedirs(directory)
		return cls(filename = directory + "/dictionary.json")

	def write(self):
		with open(self._filename, "w") as f:
			json.dump(self._entries, f, indent = 4)
			print(file = f)
			self._modified = False

	def write_if_modified(self):
		if self._modified:
			self.write()

	def add_general_word_exception(self, offense):
		self._modified = True
		if "general_words" not in self._entries:
			self._entries["general_words"] = [ ]
		self._entries["general_words"].append(offense)

	def add_word_exception(self, offense, rule_id):
		self._modified = True
		if "rule_words" not in self._entries:
			self._entries["rule_words"] = { }
		if rule_id not in self._entries["rule_words"]:
			self._entries["rule_words"][rule_id] = [ ]
		self._entries["rule_words"][rule_id].append(offense)

	def add_context_exception(self, context, rule_id):
		self._modified = True
		if "rule_context" not in self._entries:
			self._entries["rule_context"] = { }
		if rule_id not in self._entries["rule_context"]:
			self._entries["rule_context"][rule_id] = [ ]
		self._entries["rule_context"][rule_id].append(context)

	def has_exception(self, offense, context, rule_id):
		# Word is generally accepted, independent of rule/context
		if offense in self._entries.get("general_words", [ ]):
			return True

		# Word is allowed for this rule/language
		if offense in self._entries.get("rule_words", { }).get(rule_id, [ ]):
			return True

		# Context is allowed for this rule/language
		if context in self._entries.get("rule_context", { }).get(rule_id, [ ]):
			return True

		return False

	def add_exception(self, offense, context, rule_id, exception_type):
		assert(isinstance(exception_type, SpellcheckExceptionType))
		if exception_type == SpellcheckExceptionType.GeneralWord:
			self.add_general_word_exception(offense)
		elif exception_type == SpellcheckExceptionType.RuleWord:
			self.add_word_exception(offense, rule_id)
		elif exception_type == SpellcheckExceptionType.RuleContext:
			self.add_context_exception(context, rule_id)
		else:
			raise NotImplementedError(exception_type)

if __name__ == "__main__":
	dictionary = SpellcheckDictionary.open_global_dict()



#	pyradium - HTML presentation/slide show generator
#	Copyright (C) 2015-2023 Johannes Bauer
#
#	This file is part of pyradium.
#
#	pyradium is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	pyradium is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with pyradium; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import xml.dom.minidom
from .SVGPath import SVGPath
from .SVGText import SVGText

class SVGWriter():
	_NAMESPACES = {
		"inkscape": "http://www.inkscape.org/namespaces/inkscape",
		"sodipodi": "http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd",
		"svg": "http://www.w3.org/2000/svg",
		"rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
		"cc": "http://creativecommons.org/ns#",
		"dc": "http://purl.org/dc/elements/1.1/",
	}

	def __init__(self):
		self._doc = xml.dom.minidom.Document()
		self._root = self._doc.createElement("svg")
		self._root.setAttribute("xmlns", self._NAMESPACES["svg"])
		for (nsname, nsvalue) in self._NAMESPACES.items():
			self._root.setAttribute(f"xmlns:{nsname}", nsvalue)
		self._doc.appendChild(self._root)
		self._defs = None
		self._groups = { }
		self._uid = 0

	def _genid(self):
		self._uid += 1
		return f"id{self._uid}"

	def _add_definition(self, def_node):
		if self._defs is None:
			self._defs = self._doc.createElement("defs")
			self._root.appendChild(self._defs)
		def_id = self._genid()
		def_node.setAttribute("id", def_id)
		self._defs.appendChild(def_node)
		return def_id

	def group(self, group_name):
		if group_name not in self._groups:
			group = self._root.appendChild(self._doc.createElement("g"))
			group.setAttribute("inkscape:groupmode", "layer")
			group.setAttribute("inkscape:label", group_name)
			self._groups[group_name] = group
		return self._groups[group_name]

	def new_path(self, x, y, group_name = "default"):
		path_node = self._doc.createElement("path")
		self.group(group_name).appendChild(path_node)
		svg_path = SVGPath(path_node)
		svg_path.move_to(x, y)
		return svg_path

	def new_text_span(self, x, y, width, height, text, group_name = "default"):
		rect_node = self._doc.createElement("rect")
		rect_node.setAttribute("x", str(x))
		rect_node.setAttribute("y", str(y))
		rect_node.setAttribute("width", str(width))
		rect_node.setAttribute("height", str(height))
		def_id = self._add_definition(rect_node)
		text_node = self.group(group_name).appendChild(self._doc.createElement("text"))
		svg_text = SVGText(text_node)
		svg_text.text = text
		svg_text.style["shape-inside"] = f"url(#{def_id})"
		return svg_text

	def write(self, f):
		self._doc.writexml(f)

	def writefile(self, filename):
		with open(filename, "w") as f:
			self.write(f)
